from os import listdir, stat
from os.path import isdir, join, splitext, split, exists
import pygame.image
import re
GFXDIR = 'gfx'

image_cache = {} #'filename' -> Surface('gfx/filename.png')

def load_image(imagename, colorkey=True, dir=GFXDIR):
    '''Load an image, caching in image_cache.
    
    If colorkey is true, the image tries to guess the colorkey for the
    image.'''
    try:
        return image_cache[imagename]
    except KeyError:
        imagefile = join(dir, "%s.png" % imagename)
        image = pygame.image.load(imagefile)
        if colorkey:
            image = image.convert()
        image_cache[imagename] = image
        infofile = join(dir, "%s.inf"%imagename)
        if exists(infofile):
            # get colorkey from this file
            whitelist = {'None': None, 'False': False, 'True':True}
            execfile(infofile, whitelist)
            ckey = whitelist['colorkey']
            if ckey:
                try:
                    r, g, b = ckey
                except:
                    try:
                        x, y = ckey
                    except:
                        colorkey = ckey # hopefully it was T/F
                        x = y = 0
                    r, g, b, a = image.get_at((x, y))
                image.set_colorkey((r,g,b,255))
        else:
            print "No file information on", imagename, "; no colorkey set"
            
        return image

class Strip(object):
    def __init__(self, imgs, files=None):
        self.images = imgs
        self.files = files
        r = re.compile('(\d+)$')
        if len(self.images) == 1 or files == None:
            self.strip = self.images
            return
        
        def number(f):
            return int(r.search(f).group())

        lastn = number(self.files[0])
        strip = []

        for i, f in enumerate(self.files[1:]):
            n = number(f)
            strip.extend([imgs[i]]*(n-lastn))
            lastn = n
        strip.append(imgs[-1])

        self.strip = strip

    
    def __getitem__(self, n):
        return self.strip[n%len(self.strip)]

    def __len__(self):
        return len(self.strip)

    def __iter__(self):
        return iter(self.strip)

    def flip(self, fliph, flipv):
        def flip(image): return pygame.transform.flip(image, fliph, flipv)
        return Strip(map(flip, self.strip))
                     
    def rotate(self, rot):
        def rotate(image): return pygame.transform.rotate(image, rot*90)
        return Strip(map(rotate, self.strip))

    def get_width(self):
        return self.strip[0].get_width()

    def get_height(self):
        return self.strip[0].get_height()

def make_strip(filename, colorkey, dir=GFXDIR):
    if filename == '': raise ValueError, "can't read main gfx directory"
    if isdir(join(dir, filename)):
        files = listdir(join(dir,filename))
        print "HAVEN'T COVERED YET"
    else:
        l = listdir(dir)
        r = re.compile("%s(-\d+)?\.png"%filename)
        def is_part(s): return r.match(s)
        files = filter(is_part, l)
        if not files: raise ValueError, "no such file as %s in %s"%(filename, dir)
        def strip_ext(s): return splitext(s)[0]
        files = map(strip_ext, files)
    files.sort()
    def load(image): return load_image(image, colorkey, dir)
    images = map(load, files)
    return Strip(images, files=files)

def find_image(filename, location, colorkey=True):
    # Warning: if given levels/training/gfx/, will first try levels/training
    #print 'finding location at', location
    tried = []
    while location:
        location, tail = split(location)
        dir = join(location, 'gfx')
        try:
            return make_strip(filename, colorkey, dir)
        except (ValueError, OSError):
            tried.append(location)

    raise ValueError, "can't find %s (tried %s)"%(filename, tried)

