import os.path
import sys
sys.path.append(os.path.abspath('.'))
import gc

import EventManager

import View
import Scheduler
import pygame
import Input
from Vector import Vector
import PhysicsController
from Game import Game
import Level
import EnemyController
import Goals
import data.Exceptions
import MusicView
import MetaView
import Config

#pygame.init()
debug = False

if "movie" in sys.argv:
    View.movie = True
    sys.argv.remove('movie')
if "debug" in sys.argv:
    debug = True
    sys.argv.remove('debug')
try:
    levels = ["%s.py"%sys.argv[1]]
except:
    levels = [l for l in os.listdir('levels/training') if l.endswith('.py') and not l.startswith('_') and l.find('info') == -1]

if debug:
    profile = True
else:
    profile = False
musicview = MusicView.MusicView()

def play_level(levelname):
    global scheduler
    manager = EventManager.EventManager()

    g = Game(manager)
    view = View.View(g)
    scheduler = Scheduler.Scheduler()
    p = PhysicsController.PhysicsController(g)
    enemies = EnemyController.EnemyController(g)
    i = Input.Input(g)
    meta = MetaView.MetaView(g)
    
    manager.RegisterListener(view)
    manager.RegisterListener(musicview)
    levelfile = 'levels/training/%s'%levelname
    metadata = Level.metadata(levelfile)
    meta.set_info(metadata)
    scheduler.register_actor(meta, meta.thread())
    scheduler.register_actor(i, i.thread())
    try:
        Level.wait(Goals.Dialog('Blah and such; internets to the fourth degree'), g, scheduler.poll, playable=False)
    except data.Exceptions.SkipLevel:
        toret = True
    else:
        scheduler.kill_actor(meta)

        scheduler.register_actor(g, g.thread())
        scheduler.register_actor(p, p.thread())
        scheduler.register_actor(enemies, enemies.thread())
        scheduler.register_actor(scheduler, scheduler.thread(g))
        scheduler.register_actor(view, view.thread())
        try:
            Level.load(levelfile, g, scheduler.poll)
        except data.Exceptions.GameOverException:
            i.disable()
            Level.wait(Goals.RestartDialog('Press return to restart the level.'), g, scheduler.poll, playable=False)
            toret = False
        except data.Exceptions.SkipLevel:
            toret = True
        else:
            toret = 1
    scheduler.kill_all()
    view.kill_all()
    g.kill_all()
    return toret

def main():
    if debug:
        gc.enable()
        gc.set_debug(gc.DEBUG_LEAK)
    solved = {}
    try:    
        for levelname in levels:
            done = False
            while not done:
                done = play_level(levelname)
                if debug: print gc.collect(), gc.garbage
                
            if done == 1:
                Config.Config.solved[('training', os.path.splitext(levelname)[0])] = True
            
    except KeyboardInterrupt, e:
        print e
        pass
    scheduler.printstats()
    Config.Config.save_config()

if profile:
    import hotshot
    prof = hotshot.Profile('run.prof')
    prof.runcall(main)
    prof.close()
else:
    main()
